from django.db import models
from reqman.apps.account.models import User

from pygments.formatters.html import HtmlFormatter
from pygments import highlight

SUFFIX_CHOICES = (("",""), ('S1', 'S1: <numerical-armative> j <closed-interval> [per <time-unit>]'),
                   ('S2', 'S2: after/before flow'),
                   ('S3', 'S3: [every/for a period of/within/for at least] <number> <time-unit> [from flow]'),
                   ('S4', 'S4: at the beginning/at the end'),
                   ('S5', 'S5: at even intervals'),
                   ('S6', 'S6: sequentially')
                 )
NUMERICAL_AFFIRMATIVE_CHOICES = (("", ""), ("more than", "MORE THAN"), ("less than", "LESS THAN"), ("exactly","EXACTLY"), 
                                ("at least", "AT LEAST"), ("at most", "AT MOST"))
NUMBER_UNITS_CHOICES = (("",""), ("meters","METERS"), ("kilometers","KILOMETERS"), ("volt","VOLT"))
TIME_UNITS_CHOICES = (("",""), ("seconds","SECONDS"), ("minutes","MINUTES"), ("milliseconds","MILLISECONDS"))
FLOW_CHOICES = (("flow","FLOW"), ("flow2","FLOW2"))
S2_CHOICES = (("before","BEFORE"), ("after","AFTER"))
S3_CHOICES = (("every","EVERY"), ("for a period of","FOR A PERIOD OF"), ("within", "WITHIN"), ("for at least", "FOR AT LEAST"))
S4_CHOICES = (("at the beginning","AT THE BEGINNING"), ("at the end","AT THE END"))
S5_CHOICES = (("",""), ("at even intervals","AT EVEN INTERVALS"))
S6_CHOICES = (("",""), ("sequentially","SEQUENTIALLY"))


def create_suffix(instance):
    '''Create the suffix Model for the Boilerplate'''
    try :
        suffix = Suffix.objects.get(boilerplate_of_suffix = instance, suffix_owner = instance.owner)
        if instance.has_suffix:
            return
        suffix = Suffix.objects.get(boilerplate_of_suffix = instance, suffix_owner = instance.owner)
        suffix.delete()
    except Suffix.DoesNotExist:
        if instance.has_suffix:
            suffix = Suffix(boilerplate_of_suffix = instance, suffix_owner = instance.owner)
            suffix.save()


def create_s1(instance, now_suffix, prev_suffix):
    try :
        suffix = Suffix_S1.objects.get(suffix_owner_of_s1 = instance, boilerplate_of_s1 = instance.boilerplate_of_suffix)
        if now_suffix == prev_suffix:
            return
        suffix = Suffix_S1.objects.get(suffix_owner_of_s1 = instance, boilerplate_of_s1 = instance.boilerplate_of_suffix)
        suffix.delete()
        if now_suffix == SUFFIX_CHOICES[1][0]:
            suffix = Suffix_S1(suffix_owner_of_s1 = instance, boilerplate_of_s1 = instance.boilerplate_of_suffix)
            suffix.save()
    except Suffix_S1.DoesNotExist:
        if now_suffix == SUFFIX_CHOICES[1][0]:
            suffix = Suffix_S1(suffix_owner_of_s1 = instance, boilerplate_of_s1 = instance.boilerplate_of_suffix)
            suffix.save()


def create_s2(instance, now_suffix, prev_suffix):
    try :
        suffix = Suffix_S2.objects.get(suffix_owner_of_s2 = instance, boilerplate_of_s2 = instance.boilerplate_of_suffix)
        if now_suffix == prev_suffix:
            return
        suffix = Suffix_S2.objects.get(suffix_owner_of_s2 = instance, boilerplate_of_s2 = instance.boilerplate_of_suffix)
        suffix.delete()
        if now_suffix == SUFFIX_CHOICES[2][0]:
            suffix = Suffix_S2(suffix_owner_of_s2 = instance, boilerplate_of_s2 = instance.boilerplate_of_suffix)
            suffix.save()
    except Suffix_S2.DoesNotExist:
        if now_suffix == SUFFIX_CHOICES[2][0]:
            suffix = Suffix_S2(suffix_owner_of_s2 = instance, boilerplate_of_s2 = instance.boilerplate_of_suffix)
            suffix.save()


def create_s3(instance, now_suffix, prev_suffix):
    try :
        suffix = Suffix_S3.objects.get(suffix_owner_of_s3 = instance, boilerplate_of_s3 = instance.boilerplate_of_suffix)
        if now_suffix == prev_suffix:
            return
        suffix = Suffix_S3.objects.get(suffix_owner_of_s3 = instance, boilerplate_of_s3 = instance.boilerplate_of_suffix)
        suffix.delete()
        if now_suffix == SUFFIX_CHOICES[3][0]:
            suffix = Suffix_S3(suffix_owner_of_s3 = instance, boilerplate_of_s3 = instance.boilerplate_of_suffix)
            suffix.save()
    except Suffix_S3.DoesNotExist:
        if now_suffix == SUFFIX_CHOICES[3][0]:
            suffix = Suffix_S3(suffix_owner_of_s3 = instance, boilerplate_of_s3 = instance.boilerplate_of_suffix)
            suffix.save()


def create_s4(instance, now_suffix, prev_suffix):
    try :
        suffix = Suffix_S4.objects.get(suffix_owner_of_s4 = instance, boilerplate_of_s4 = instance.boilerplate_of_suffix)
        if now_suffix == prev_suffix:
            return
        suffix = Suffix_S4.objects.get(suffix_owner_of_s4 = instance, boilerplate_of_s4 = instance.boilerplate_of_suffix)
        suffix.delete()
        if now_suffix == SUFFIX_CHOICES[4][0]:
            suffix = Suffix_S4(suffix_owner_of_s4 = instance, boilerplate_of_s4 = instance.boilerplate_of_suffix)
            suffix.save()
    except Suffix_S4.DoesNotExist:
        if now_suffix == SUFFIX_CHOICES[4][0]:
            suffix = Suffix_S4(suffix_owner_of_s4 = instance, boilerplate_of_s4 = instance.boilerplate_of_suffix)
            suffix.save()


def create_s5(instance, now_suffix, prev_suffix):
    try :
        suffix = Suffix_S5.objects.get(suffix_owner_of_s5 = instance, boilerplate_of_s5 = instance.boilerplate_of_suffix)
        if now_suffix == prev_suffix:
            return
        suffix = Suffix_S5.objects.get(suffix_owner_of_s5 = instance, boilerplate_of_s5 = instance.boilerplate_of_suffix)
        suffix.delete()
        if now_suffix == SUFFIX_CHOICES[5][0]:
            suffix = Suffix_S5(suffix_owner_of_s5 = instance, boilerplate_of_s5 = instance.boilerplate_of_suffix)
            suffix.save()
    except Suffix_S5.DoesNotExist:
        if now_suffix == SUFFIX_CHOICES[5][0]:
            suffix = Suffix_S5(suffix_owner_of_s5 = instance, boilerplate_of_s5 = instance.boilerplate_of_suffix)
            suffix.save()


def create_s6(instance, now_suffix, prev_suffix):
    try :
        suffix = Suffix_S6.objects.get(suffix_owner_of_s6 = instance, boilerplate_of_s6 = instance.boilerplate_of_suffix)
        if now_suffix == prev_suffix:
            return
        suffix = Suffix_S6.objects.get(suffix_owner_of_s6 = instance, boilerplate_of_s6 = instance.boilerplate_of_suffix)
        suffix.delete()
        if now_suffix == SUFFIX_CHOICES[6][0]:
            suffix = Suffix_S6(suffix_owner_of_s6 = instance, boilerplate_of_s6 = instance.boilerplate_of_suffix)
            suffix.save()
    except Suffix_S6.DoesNotExist:
        if now_suffix == SUFFIX_CHOICES[6][0]:
            suffix = Suffix_S6(suffix_owner_of_s6 = instance, boilerplate_of_s6 = instance.boilerplate_of_suffix)
            suffix.save()



class Suffix(models.Model):
    boilerplate_of_suffix = models.ForeignKey('reqtool.Boilerplate', related_name='suffix_boilerplate', on_delete=models.CASCADE)
    suffix_owner = models.ForeignKey(User, related_name='suffix_owner', on_delete=models.CASCADE)
    suffix_choices = models.CharField(choices=SUFFIX_CHOICES, max_length=100)

    

    def __init__(self, *args, **kwargs):
        super(Suffix, self).__init__(*args, **kwargs)
        self.__important_fields = ['suffix_choices']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))
        

    def save(self, **kwargs):
        prev_suffix_list = []
        super(Suffix, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_suffix = getattr(self, orig)
            prev_suffix_list.append(previous_suffix)        
        '''Create [S1 - S6]'''
        create_s1(self, self.suffix_choices, prev_suffix_list[0])
        create_s2(self, self.suffix_choices, prev_suffix_list[0])
        create_s3(self, self.suffix_choices, prev_suffix_list[0])
        create_s4(self, self.suffix_choices, prev_suffix_list[0])
        create_s5(self, self.suffix_choices, prev_suffix_list[0])
        create_s6(self, self.suffix_choices, prev_suffix_list[0])



    class Meta:
        ordering = ['boilerplate_of_suffix']


class Suffix_S1(models.Model):
    boilerplate_of_s1 = models.ForeignKey('reqtool.Boilerplate', related_name='suffix_boilerplate_of_S1', on_delete=models.CASCADE)
    suffix_owner_of_s1 = models.ForeignKey(Suffix, related_name='suffix_owner_of_S1', on_delete=models.CASCADE)
    numerical_affirmative_s1 = models.CharField(choices=NUMERICAL_AFFIRMATIVE_CHOICES, max_length=100)
    numerical_s1 = models.IntegerField(default=0, blank=True) 
    mumerical_units_s1 = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    time_units_s1 = models.CharField(choices=TIME_UNITS_CHOICES, max_length=100)
         

    class Meta:
        ordering = ['boilerplate_of_s1']


class Suffix_S2(models.Model):
    boilerplate_of_s2 = models.ForeignKey('reqtool.Boilerplate', related_name='suffix_boilerplate_of_S2', on_delete=models.CASCADE)
    suffix_owner_of_s2 = models.ForeignKey(Suffix, related_name='suffix_owner_of_S2', on_delete=models.CASCADE)
    s2_s2 = models.CharField(choices=S2_CHOICES, max_length=100)
    flow_s2 = models.CharField(choices=FLOW_CHOICES, max_length=100)
         

    class Meta:
        ordering = ['boilerplate_of_s2']


class Suffix_S3(models.Model):
    boilerplate_of_s3 = models.ForeignKey('reqtool.Boilerplate', related_name='suffix_boilerplate_of_S3', on_delete=models.CASCADE)
    suffix_owner_of_s3 = models.ForeignKey(Suffix, related_name='suffix_owner_of_S3', on_delete=models.CASCADE)
    s3_s3 = models.CharField(choices=S3_CHOICES, max_length=100)
    numerical_s3 = models.IntegerField(default=0, blank=True) 
    time_units_s3 = models.CharField(choices=TIME_UNITS_CHOICES, max_length=100)
    flow_s3 = models.CharField(choices=FLOW_CHOICES, max_length=100)
         

    class Meta:
        ordering = ['boilerplate_of_s3']

class Suffix_S4(models.Model):
    boilerplate_of_s4 = models.ForeignKey('reqtool.Boilerplate', related_name='suffix_boilerplate_of_S4', on_delete=models.CASCADE)
    suffix_owner_of_s4 = models.ForeignKey(Suffix, related_name='suffix_owner_of_S4', on_delete=models.CASCADE)
    s4_s4 = models.CharField(choices=S4_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_s4']


class Suffix_S5(models.Model):
    boilerplate_of_s5 = models.ForeignKey('reqtool.Boilerplate', related_name='suffix_boilerplate_of_S5', on_delete=models.CASCADE)
    suffix_owner_of_s5 = models.ForeignKey(Suffix, related_name='suffix_owner_of_S5', on_delete=models.CASCADE)
    s5_s5 = models.CharField(choices=S5_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_s5']


class Suffix_S6(models.Model):
    boilerplate_of_s6 = models.ForeignKey('reqtool.Boilerplate', related_name='suffix_boilerplate_of_S6', on_delete=models.CASCADE)
    suffix_owner_of_s6 = models.ForeignKey(Suffix, related_name='suffix_owner_of_S6', on_delete=models.CASCADE)
    s6_s6 = models.CharField(choices=S6_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_s6']