from rest_framework import serializers
from reqman.apps.reqtool.models.requirements import Prefix, Boilerplate, LogicalExpression, OccuringFunctionalityP2, StateValueConstraint, OccuringFunctionalityForLogicalExpression, OccuringFunctionalityP3

class BoilerplateSerializer(serializers.ModelSerializer):
    owner = serializers.ReadOnlyField(source='owner.username')
    class Meta:
        model = Boilerplate
        fields = ('id', 'owner', 'has_prefix', 'has_main', 'has_suffix')


class PrefixSerializer(serializers.ModelSerializer):
    boilerplate = serializers.PrimaryKeyRelatedField(read_only=True)
    prefix_owner = serializers.PrimaryKeyRelatedField(read_only=True)
    class Meta:
        model = Prefix
        fields = ( 'id', 'prefix_owner', 'boilerplate', 'simple_prefix_1', 'logic_connective_1', 'simple_prefix_2', 'logic_connective_2', 'simple_prefix_3')

class LogicalExpressionSerializer(serializers.ModelSerializer):
    prefix = serializers.PrimaryKeyRelatedField(read_only=True)
    prefix_boilerplate = serializers.PrimaryKeyRelatedField(read_only=True)
    prefix_series = serializers.PrimaryKeyRelatedField(read_only=True)
    
    class Meta:
        model = LogicalExpression
        fields = ('id', 'prefix', 'prefix_series', 'prefix_boilerplate', 'logical_expression_1', 'logic_connective_1', 'logical_expression_2', 'logic_connective_2', 'logical_expression_3')


class OccuringFunctionalityP2Serializer(serializers.ModelSerializer):
    prefix_of_occuring_functionality_p2 = serializers.PrimaryKeyRelatedField(read_only=True)
    prefix_boilerplate_of_occuring_functionality_p2 = serializers.PrimaryKeyRelatedField(read_only=True)
    prefix_series_of_occuring_functionality_p2 = serializers.PrimaryKeyRelatedField(read_only=True)
    class Meta:
        model = OccuringFunctionalityP2
        fields = ('id', 'prefix_of_occuring_functionality_p2', 'prefix_boilerplate_of_occuring_functionality_p2', 'prefix_series_of_occuring_functionality_p2', 'system_for_occuring_functionality_p2', 'perform_for_occuring_functionality_p2', 'function_for_occuring_functionality_p2', 'item_for_occuring_functionality_p2', 'flow_for_occuring_functionality_p2')



class OccuringFunctionalityP3Serializer(serializers.ModelSerializer):
    prefix_of_occuring_functionality_p3 = serializers.PrimaryKeyRelatedField(read_only=True)
    prefix_boilerplate_of_occuring_functionality_p3 = serializers.PrimaryKeyRelatedField(read_only=True)
    prefix_series_of_occuring_functionality_p3 = serializers.PrimaryKeyRelatedField(read_only=True)
    class Meta:
        model = OccuringFunctionalityP3
        fields = ('id', 'prefix_of_occuring_functionality_p3', 'prefix_boilerplate_of_occuring_functionality_p3', 'prefix_series_of_occuring_functionality_p3', 'system_for_occuring_functionality_p3', 'perform_for_occuring_functionality_p3', 'function_for_occuring_functionality_p3', 'item_for_occuring_functionality_p3', 'flow_for_occuring_functionality_p3')



class StateValueConstraintSerializer(serializers.ModelSerializer):
    prefix_of_logical_expression = serializers.PrimaryKeyRelatedField(read_only=True)
    logical_expression = serializers.PrimaryKeyRelatedField(read_only=True)
    log_expr_series = serializers.PrimaryKeyRelatedField(read_only=True)
    class Meta:
        model = StateValueConstraint
        fields = ('id', 'prefix_of_logical_expression', 'logical_expression', 'log_expr_series', 'item', 'system_state', 'state_value')


class OccuringFunctionalityForLogicalExpressionSerializer(serializers.ModelSerializer):
    prefix_of_occuring_for_logical_expression = serializers.PrimaryKeyRelatedField(read_only=True)
    logical_expression_of_occuring = serializers.PrimaryKeyRelatedField(read_only=True)
    log_expres_series = serializers.PrimaryKeyRelatedField(read_only=True)
    class Meta:
        model = OccuringFunctionalityForLogicalExpression
        fields = ('id', 'prefix_of_occuring_for_logical_expression', 'logical_expression_of_occuring', 'log_expres_series', 'system_for_logical_expression', 'perform_for_logical_expression', 'function_for_logical_expression', 'item_for_logical_expression')
